import com.visibleworkings.trace.TraceLogList;
import com.visibleworkings.trace.TraceParsedMessage;
import java.io.File;
import java.util.List;


public class CheckResults extends LogChecker {

    static protected void failureTrailer() {
        System.out.println("See the installation instructions for what to do now.");
    }

    static protected void success() { 
        System.out.println("##############################################################");
        System.out.println("############################ SUCCESS #########################");
        System.out.println("##############################################################");
        System.exit(0);
    }

    private static void checkTraceLog(File cwd) {
        pushContext("... checking Tracelog.txt");

        TraceLogList list = openLogs(cwd, "Tracelog");
        assertNumLogs(list, 1);
        assertLastLogName(list, "Tracelog.txt");

        List msgs = logMessages(list);
        assertNumMessages(msgs, 4);
        
        assertValues((TraceParsedMessage) msgs.get(0),
            "InstallTest.main", "InstallTest.java", "18", "ERR",
            "InstallTest", "Here is a trace message at the ERROR level.");
        assertValues((TraceParsedMessage) msgs.get(1),
            "InstallTest.main", "InstallTest.java", "19", "WRN",
            "InstallTest", "Here is a trace message at the WARNING level.");
        assertValues((TraceParsedMessage) msgs.get(2),
            "InstallTest.main", "InstallTest.java", "20", "WLD",
            "InstallTest", "Here is a trace message at the WORLD level.");
        assertValues((TraceParsedMessage) msgs.get(3),
            "InstallTest.main", "InstallTest.java", "32", "WLD",
            "InstallTest", "I choose to send further messages to a timestamped file.");
        popContext();
    }

    private static void checkNewName(File cwd) {
        pushContext("... checking TracelogNewName files.");

        TraceLogList list = openLogs(cwd, "TracelogNewName");
        assertNumLogs(list, 2);

        List msgs = logMessages(list);
        assert(msgs.size() >= 31, "The two TracelogNewName files should have 31 or more messages.");

        assertValues((TraceParsedMessage) msgs.get(msgs.size()-1),
            "InstallTest.main", "InstallTest.java", "56", "WLD",
            "InstallTest", "Thanks for installing Trace.java.");

        popContext(); // done with TracelogNewName.
    }
    
        
    public static void main(String[] args) {

        pushContext("... checking trace logs.");
        File cwd = new File(System.getProperty("user.dir"));

        checkTraceLog(cwd);
        checkNewName(cwd);

        popContext();
        success();
    }

}
