import com.visibleworkings.trace.Trace;
import com.visibleworkings.trace.TraceController;
import java.util.Properties;

public class InstallTest {

    // All trace messages from this "subsystem" go to this trace object.
    // A subsystem is usually a collection of related classes, perhaps
    // all the classes in a package.
    static private Trace tr = new Trace("InstallTest");

    public static void main(String[] args) {
        TraceController.start();

        // Uncomment this if the installation test fails.
        // TraceController.debugTraceCaller();

        tr.errorm("Here is a trace message at the ERROR level.");
        tr.warningm("Here is a trace message at the WARNING level.");
        tr.worldm("Here is a trace message at the WORLD level.");

        tr.usagem("This USAGE message is at too low a level to appear in the log.");
        tr.usagem("But it will appear in the ring buffer.");

        tr.eventm("This EVENT message will appear in neither the disk log nor ring buffer.");
        tr.debugm("This DEBUG message will appear in neither the disk log nor ring buffer.");
        tr.verbosem("This VERBOSE message will appear in neither the disk log nor ring buffer.");

        // I think I'd rather log to a timestamped file.
        // So we see how log files fill up, make the allowable size
        // small. 
        tr.worldm("I choose to send further messages to a timestamped file.");
        TraceController.setProperty("TraceLog_tag", "TracelogNewName");
        TraceController.setProperty("TraceLog_timestamp", "true");
        TraceController.setProperty("TraceLog_size", "1000");
        TraceController.setProperty("TraceLog_reopen", "true");

        tr.worldm("Let's put things in the log so that it fills up!");
        for(int i = 0; i < 10; i++) {
            // Note that you can tack on an object, which Trace prints
            // with toString().
            tr.worldm("Here's some text that will help to fill it up.",
                new Integer(i));
        }

        // Let us now dump the internal ring buffer. You'll see that
        // it contains everything that was dumped to the log, plus the
        // lower level ("debug") messages as well. 
        // But first, let's up the log size so that we don't get yet
        // more log files. 
        TraceController.setProperty("TraceLog_size", "unlimited");
        TraceController.dumpBufferToLog();

        tr.warningm("Some VMs mess up the stack by this point.");
        tr.worldm("If this line is wrong, please report it.");
        tr.worldm("Thanks for installing Trace.java.");
    }
}
