package com.visibleworkings.trace;

import junit.framework.*;
import java.util.Date;
import java.util.Locale;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class TestTraceDateToString extends TestCase {


	public TestTraceDateToString(String name) {
		super(name);
	}

	protected void setUp() {
	}

	public static Test suite() {
		return new TestSuite(TestTraceDateToString.class);
	}


    public void testZeroFilling() {
        // Use the epoch to test zero filling. Have to use Calendar to
        // take into account current time zone.  Note that zero
        // filling of milliseconds can't be tested this way because
        // you can't set the millisecond field of a Calendar, so it
        // seems to be filled with garbage (though the JDK1.2.2
        // documentation suggests it should be 0).
        Calendar calendar = new GregorianCalendar();
        calendar.set(1970, Calendar.JANUARY, 2, 0, 0, 0); 
        Date epoch = calendar.getTime();
        assertEquals("1970/01/02",
                     TraceDateToString.dateString(epoch));
        assertEquals("19700102000000",
                     TraceDateToString.terseCompleteDateString(epoch));
    }

    public void testTypicalDate() {
        Locale enUS = new Locale("en", "US");
        SimpleDateFormat format =
            new SimpleDateFormat("yyyy MM dd HH mm ss SSS");
        Date date = format.parse("2001 12 31 12 34 59 123", new ParsePosition(0));

        assertEquals("2001/12/31",
                     TraceDateToString.dateString(date));
        assertEquals("20011231123459",
                     TraceDateToString.terseCompleteDateString(date));
        assertEquals("12:34:59.123",
                     TraceDateToString.timeString(date));
    }

    public void testZeroFilledTypicalDate() {
        Locale enUS = new Locale("en", "US");
        SimpleDateFormat format =
            new SimpleDateFormat("yyyy MM dd HH mm ss SSS");
        Date date = format.parse("2001 01 02 03 05 06 000", new ParsePosition(0));

        assertEquals("2001/01/02",
                     TraceDateToString.dateString(date));
        assertEquals("20010102030506",
                     TraceDateToString.terseCompleteDateString(date));
        assertEquals("03:05:06.000",
                     TraceDateToString.timeString(date));
    }

}
