package com.visibleworkings.trace;

/*
The contents of this file are subject to the Electric Communities E Open 
Source Code License Version 1.0 (the "License"); you may not use this file 
except in compliance with the License. You may obtain a copy of the License 
at http://www.communities.com/EL/.

Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for 
the specific language governing rights and limitations under the License.

The Original Code is the Distributed E Language Implementation, released 
July 20, 1998. 

The Initial Developer of the Original Code is Electric Communities.  
Copyright (C) 1998 Electric Communities. All Rights Reserved. 

Contributor(s): ______________________________________.
*/
/*
 *  Trace and Logging Package.  Written by Brian Marick,
 *  July-September 1997, for Electric Communities, Inc.
 *  Copyright (c) 1997 by Electric Communities.  All Rights Reserved.
 */


import java.util.Date;
import java.util.Locale;
import java.text.SimpleDateFormat;

/** 
 * Convert java.util.Dates into suitable Strings for the Trace system.
 * <p>
 * Note that times are printed in local time rather than GMT. People
 * get confused when looking at log files that are not in the local
 * time - they know something bad happened at 4 o'clock, but there's
 * nothing in the log for that time.
 * <p>
 * For synchronizing trace logs from different time zones, this is
 * inadequate. We'll deal with that when we come to it.
 * <p>
 * Note that the construction of strings in this class is one of the
 * bottlenecks in this code.
 */
class TraceDateToString {
    // Although all formatting is locale-independent (I hope),
    // SimpleDateFormat is not supported in all locales. So we
    // explicitly use a Culturally Hegemonic Locale that is
    // supported. 
    private static Locale theCHL = new Locale("en", "US");

    // Here are the formats. See the method descriptions for more.

    private static SimpleDateFormat theDateFormatter =
        new SimpleDateFormat("yyyy/MM/dd", theCHL);

    private static SimpleDateFormat theCompleteFormatter =
        new SimpleDateFormat("yyyyMMddHHmmss", theCHL);

    private static SimpleDateFormat theTimeFormatter =
        new SimpleDateFormat("HH:mm:ss.SSS", theCHL);


    /** 
     * Returns a string like 1997/04/23.  Note that strings of this
     * form can be sorted by date, unlike M/D/Y or D/M/Y or forms that
     * aren't zero-padded.  By being in obviously non-European and
     * non-US order (because the year is first), no one gets confused
     * as to which field is the day and which is the month.
     */
    /*package*/ final static String dateString (Date date) {
        return theDateFormatter.format(date);
    }


    /**
     * Return time in form YYYYMMDDHHMMSS.  This is a terse sortable
     * time.  Fields are zero-padded as needed.
     */
    /*package*/ final static String terseCompleteDateString(Date date) {
        return theCompleteFormatter.format(date);
    }


    /**
     * Return time in form HH:MM:SS.mmm.  Fields are zero-padded as
     * needed.
     */
    /*package*/ final static String timeString (Date date) {
        return theTimeFormatter.format(date);
    }
}
